@@ -1,9 +1,10 @@
 use std::convert::TryInto;
-use std::path::{Path, PathBuf};
+use std::ffi::OsString;
+use std::path::PathBuf;
 
 use anyhow::Context;
 use branchless::commands::wrap;
-use branchless::util::GitExecutable;
+use branchless::util::GitRunInfo;
 use simple_logger::SimpleLogger;
 use structopt::StructOpt;
 
@@ -17,10 +18,14 @@ enum WrappedCommand {
 ///
 /// See the documentation at https://github.com/arxanas/git-branchless/wiki.
 #[derive(StructOpt)]
-#[structopt(version = "0.2.0", author = "Waleed Khan <me@waleedkhan.name>")]
+#[structopt(version = env!("CARGO_PKG_VERSION"), author = "Waleed Khan <me@waleedkhan.name>")]
 enum Opts {
     /// Initialize the branchless workflow for this repository.
-    Init,
+    Init {
+        /// Uninstall the branchless workflow instead of initializing it.
+        #[structopt(long = "--uninstall")]
+        uninstall: bool,
+    },
 
     /// Display a nice graph of the commits you've recently worked on.
     Smartlog,
@@ -143,13 +148,22 @@ fn main() -> anyhow::Result<()> {
         .with_context(|| "Initializing logging")?;
 
     let opts = Opts::from_args();
-    let git_executable = std::env::var("PATH_TO_GIT").unwrap_or_else(|_| "git".to_string());
-    let git_executable = Path::new(&git_executable);
-    let git_executable = GitExecutable(git_executable.to_path_buf());
+    let path_to_git = std::env::var_os("PATH_TO_GIT").unwrap_or_else(|| OsString::from("git"));
+    let path_to_git = PathBuf::from(&path_to_git);
+    let git_run_info = GitRunInfo {
+        path_to_git,
+        working_directory: std::env::current_dir()?,
+        env: std::env::vars_os().collect(),
+    };
 
     let exit_code = match opts {
-        Opts::Init => {
-            branchless::commands::init::init(&git_executable)?;
+        Opts::Init { uninstall: false } => {
+            branchless::commands::init::init(&git_run_info)?;
+            0
+        }
+
+        Opts::Init { uninstall: true } => {
+            branchless::commands::init::uninstall()?;
             0
         }
 
@@ -165,7 +179,7 @@ fn main() -> anyhow::Result<()> {
         }
 
         Opts::Prev { num_commits } => {
-            branchless::commands::navigation::prev(&&git_executable, num_commits)?
+            branchless::commands::navigation::prev(&&git_run_info, num_commits)?
         }
 
         Opts::Next {
@@ -179,7 +193,7 @@ fn main() -> anyhow::Result<()> {
                 (false, true) => Some(branchless::commands::navigation::Towards::Newest),
                 (true, true) => anyhow::bail!("Both --oldest and --newest were set"),
             };
-            branchless::commands::navigation::next(&git_executable, num_commits, towards)?
+            branchless::commands::navigation::next(&git_run_info, num_commits, towards)?
         }
 
         Opts::Move {
@@ -187,17 +201,13 @@ fn main() -> anyhow::Result<()> {
             dest,
             base,
             force_on_disk,
-        } => branchless::commands::r#move::r#move(
-            &git_executable,
-            source,
-            dest,
-            base,
-            force_on_disk,
-        )?,
+        } => {
+            branchless::commands::r#move::r#move(&git_run_info, source, dest, base, force_on_disk)?
+        }
 
-        Opts::Restack => branchless::commands::restack::restack(&git_executable)?,
+        Opts::Restack => branchless::commands::restack::restack(&git_run_info)?,
 
-        Opts::Undo => branchless::commands::undo::undo(&git_executable)?,
+        Opts::Undo => branchless::commands::undo::undo(&git_run_info)?,
 
         Opts::Gc | Opts::HookPreAutoGc => {
             branchless::commands::gc::gc()?;
@@ -208,11 +218,14 @@ fn main() -> anyhow::Result<()> {
             git_executable: explicit_git_executable,
             command: WrappedCommand::WrappedCommand(args),
         } => {
-            let git_executable = match explicit_git_executable {
-                Some(path) => GitExecutable(path),
-                None => git_executable,
+            let git_run_info = match explicit_git_executable {
+                Some(path_to_git) => GitRunInfo {
+                    path_to_git,
+                    ..git_run_info
+                },
+                None => git_run_info,
             };
-            wrap::wrap(&git_executable, args.as_slice())?;
+            wrap::wrap(&git_run_info, args.as_slice())?;
             0
         }
 
