@@ -3,13 +3,16 @@
 //! This is inside `src` rather than `tests` since we use this code in some unit
 //! tests.
 
-use std::path::{Path, PathBuf};
-use std::process::Command;
-use std::str::FromStr;
+use std::ffi::{OsStr, OsString};
+use std::io::Write;
+use std::ops::Deref;
+use std::path::PathBuf;
+use std::process::{Command, Stdio};
 
-use crate::util::{get_sh, wrap_git_error, GitExecutable, GitVersion};
+use crate::util::{get_sh, wrap_git_error, GitRunInfo, GitVersion};
 use anyhow::Context;
 use fn_error_context::context;
+use tempfile::TempDir;
 
 const DUMMY_NAME: &str = "Testy McTestface";
 const DUMMY_EMAIL: &str = "test@example.com";
@@ -25,7 +28,7 @@ pub struct Git {
 
     /// The path to the Git executable on disk. This is important since we test
     /// against multiple Git versions.
-    pub git_executable: PathBuf,
+    pub path_to_git: PathBuf,
 }
 
 /// Options for `Git::init_repo_with_options`.
@@ -34,12 +37,16 @@ pub struct GitInitOptions {
     /// If `true`, then `init_repo_with_options` makes an initial commit with
     /// some content.
     pub make_initial_commit: bool,
+
+    /// If `true`, run `git branchless init` as part of initialization process.
+    pub run_branchless_init: bool,
 }
 
 impl Default for GitInitOptions {
     fn default() -> Self {
         GitInitOptions {
             make_initial_commit: true,
+            run_branchless_init: true,
         }
     }
 }
@@ -54,9 +61,8 @@ pub struct GitRunOptions {
     /// The exit code that `Git` should return.
     pub expected_exit_code: i32,
 
-    /// If `true`, use the system installation of `git` for this command, rather
-    /// than the version under test.
-    pub use_system_git: bool,
+    /// The input to write to the child process's stdin.
+    pub input: Option<String>,
 }
 
 impl Default for GitRunOptions {
@@ -64,39 +70,45 @@ impl Default for GitRunOptions {
         GitRunOptions {
             time: 0,
             expected_exit_code: 0,
-            use_system_git: false,
+            input: None,
         }
     }
 }
 
 impl Git {
     /// Constructor.
-    pub fn new(repo_path: PathBuf, git_executable: GitExecutable) -> Self {
-        let GitExecutable(git_executable) = git_executable;
+    pub fn new(repo_path: PathBuf, git_run_info: GitRunInfo) -> Self {
+        let GitRunInfo {
+            path_to_git,
+            // We pass the repo directory when calling `run`.
+            working_directory: _,
+            // We manually set the environment when calling `run`.
+            env: _,
+        } = git_run_info;
         Git {
             repo_path,
-            git_executable,
+            path_to_git,
         }
     }
 
     /// Replace dynamic strings in the output, for testing purposes.
     pub fn preprocess_stdout(&self, stdout: String) -> anyhow::Result<String> {
-        let git_executable = self
-            .git_executable
+        let path_to_git = self
+            .path_to_git
             .to_str()
-            .ok_or_else(|| anyhow::anyhow!("Could not convert Git executable to string"))?;
-        let stdout = stdout.replace(git_executable, "<git-executable>");
+            .ok_or_else(|| anyhow::anyhow!("Could not convert path to Git to string"))?;
+        let stdout = stdout.replace(path_to_git, "<git-executable>");
         Ok(stdout)
     }
 
     /// Get the `PATH` environment variable to use for testing.
-    pub fn get_path_for_env(&self) -> String {
+    pub fn get_path_for_env(&self) -> OsString {
         let cargo_bin_path = assert_cmd::cargo::cargo_bin("git-branchless");
         let branchless_path = cargo_bin_path
             .parent()
             .expect("Unable to find git-branchless path parent");
         let git_path = self
-            .git_executable
+            .path_to_git
             .parent()
             .expect("Unable to find git path parent");
         let bash = get_sh().expect("bash missing?");
@@ -114,8 +126,6 @@ impl Git {
             .map(|path| path.to_str().expect("Unable to decode path component")),
         )
         .expect("joining paths")
-        .to_string_lossy()
-        .to_string()
     }
 
     /// Run a Git command.
@@ -126,25 +136,14 @@ impl Git {
         options: &GitRunOptions,
     ) -> anyhow::Result<(String, String)> {
         let GitRunOptions {
-            use_system_git,
             time,
             expected_exit_code,
+            input,
         } = options;
 
-        let default_path = if cfg!(target_os = "windows") {
-            Path::new("C:\\Program Files\\Git\\bin\\git.exe")
-        } else {
-            Path::new("/usr/bin/git")
-        };
-        let git_executable = if !use_system_git {
-            &self.git_executable
-        } else {
-            default_path
-        };
-
         // Required for determinism, as these values will be baked into the commit
         // hash.
-        let date = format!("{date} -{time:0>2}", date = DUMMY_DATE, time = time);
+        let date: OsString = format!("{date} -{time:0>2}", date = DUMMY_DATE, time = time).into();
 
         let args: Vec<&str> = {
             let repo_path = self.repo_path.to_str().expect("Could not decode repo path");
@@ -153,38 +152,54 @@ impl Git {
             new_args
         };
 
+        // Fake "editor" which accepts the default contents of any commit
+        // messages. Usually, we can set this with `git commit -m`, but we have
+        // no such option for things such as `git rebase`, which may call `git
+        // commit` later as a part of their execution.
+        //
+        // ":" is understood by `git` to skip editing.
+        let git_editor = OsString::from(":");
+
         let new_path = self.get_path_for_env();
-        let env: Vec<(&str, &str)> = vec![
+        let env: Vec<(&str, &OsStr)> = vec![
             ("GIT_AUTHOR_DATE", &date),
             ("GIT_COMMITTER_DATE", &date),
-            // Fake "editor" which accepts the default contents of any commit
-            // messages. Usually, we can set this with `git commit -m`, but we have
-            // no such option for things such as `git rebase`, which may call `git
-            // commit` later as a part of their execution.
-            //
-            // ":" is understood by `git` to skip editing.
-            ("GIT_EDITOR", ":"),
-            (
-                "PATH_TO_GIT",
-                git_executable
-                    .to_str()
-                    .expect("Could not decode `git_executable`"),
-            ),
+            ("GIT_EDITOR", &git_editor),
+            ("PATH_TO_GIT", &self.path_to_git.as_os_str()),
             ("PATH", &new_path),
         ];
 
-        let mut command = Command::new(&git_executable);
+        let mut command = Command::new(&self.path_to_git);
         command.args(&args).env_clear().envs(env.iter().copied());
 
-        let result = command.output().with_context(|| {
-            format!(
-                "Running git
-                Executable: {:?}
-                Args: {:?}
-                Env: <not shown>",
-                &self.git_executable, &args
-            )
-        })?;
+        let result = if let Some(input) = input {
+            let mut child = command
+                .stdin(Stdio::piped())
+                .stdout(Stdio::piped())
+                .stderr(Stdio::piped())
+                .spawn()?;
+            write!(child.stdin.take().unwrap(), "{}", &input)?;
+            child.wait_with_output().with_context(|| {
+                format!(
+                    "Running git
+                    Executable: {:?}
+                    Args: {:?}
+                    Stdin: {:?}
+                    Env: <not shown>",
+                    &self.path_to_git, &args, input
+                )
+            })?
+        } else {
+            command.output().with_context(|| {
+                format!(
+                    "Running git
+                    Executable: {:?}
+                    Args: {:?}
+                    Env: <not shown>",
+                    &self.path_to_git, &args
+                )
+            })?
+        };
 
         let exit_code = result
             .status
@@ -197,7 +212,7 @@ impl Git {
                 {}
                 stderr:
                 {}",
-                &self.git_executable,
+                &self.path_to_git,
                 &args,
                 exit_code,
                 expected_exit_code,
@@ -236,7 +251,9 @@ impl Git {
         // Non-deterministic metadata (depends on current time).
         self.run(&["config", "branchless.commitMetadata.relativeTime", "false"])?;
 
-        self.run(&["branchless", "init"])?;
+        if options.run_branchless_init {
+            self.run(&["branchless", "init"])?;
+        }
 
         Ok(())
     }
@@ -336,19 +353,40 @@ impl Git {
 
 /// Get the path to the Git executable for testing.
 #[context("Getting the Git executable to use")]
-pub fn get_git_executable() -> anyhow::Result<PathBuf> {
-    let git_executable = std::env::var("PATH_TO_GIT").with_context(|| {
+pub fn get_path_to_git() -> anyhow::Result<PathBuf> {
+    let path_to_git = std::env::var_os("PATH_TO_GIT").with_context(|| {
         "No path to git set. Try running as: PATH_TO_GIT=$(which git) cargo test ..."
     })?;
-    let git_executable = PathBuf::from_str(&git_executable)?;
-    Ok(git_executable)
+    let path_to_git = PathBuf::from(&path_to_git);
+    Ok(path_to_git)
+}
+
+/// Wrapper around a `Git` instance which cleans up the repository once dropped.
+pub struct GitWrapper {
+    _repo_dir: TempDir,
+    git: Git,
+}
+
+impl Deref for GitWrapper {
+    type Target = Git;
+
+    fn deref(&self) -> &Self::Target {
+        &self.git
+    }
 }
 
 /// Create a temporary directory for testing and a `Git` instance to use with it.
-pub fn with_git(f: fn(Git) -> anyhow::Result<()>) -> anyhow::Result<()> {
+pub fn make_git() -> anyhow::Result<GitWrapper> {
     let repo_dir = tempfile::tempdir()?;
-    let git_executable = get_git_executable()?;
-    let git_executable = GitExecutable(git_executable);
-    let git = Git::new(repo_dir.path().to_path_buf(), git_executable);
-    f(git)
+    let path_to_git = get_path_to_git()?;
+    let path_to_git = GitRunInfo {
+        path_to_git,
+        working_directory: repo_dir.path().to_path_buf(),
+        env: std::env::vars_os().collect(),
+    };
+    let git = Git::new(repo_dir.path().to_path_buf(), path_to_git);
+    Ok(GitWrapper {
+        _repo_dir: repo_dir,
+        git,
+    })
 }
